!> @file user_3d_data_averaging.f90
!------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2017 Leibniz Universitaet Hannover
!------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_3d_data_averaging.f90 2101 2017-01-05 16:42:31Z suehring $
!
! 2000 2016-08-20 18:09:15Z knoop
! Forced header and separation lines into 80 columns
! 
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1353 2014-04-08 15:21:23Z heinze
! REAL constants provided with KIND-attribute
!
! 1322 2014-03-20 16:38:49Z raasch
! REAL functions provided with KIND-attribute
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements,
! kinds are defined in new module kinds,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Sum up and time-average user-defined output quantities as well as allocate
!> the array necessary for storing the average.
!------------------------------------------------------------------------------!
 SUBROUTINE user_3d_data_averaging( mode, variable )
 

    USE control_parameters

    USE indices

    USE kinds

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  mode    !< 
    CHARACTER (LEN=*) :: variable !< 

    INTEGER(iwp) ::  i !< 
    INTEGER(iwp) ::  j !< 
    INTEGER(iwp) ::  k !< 

    IF ( mode == 'allocate' )  THEN

       SELECT CASE ( TRIM( variable ) )

          CASE ( 'usss_r' )
             IF ( .NOT. ALLOCATED( usss_res_av ) )  THEN
                ALLOCATE( usss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             usss_res_av = 0.0_wp

          CASE ( 'vsss_r' )
             IF ( .NOT. ALLOCATED( vsss_res_av ) )  THEN
                ALLOCATE( vsss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             vsss_res_av = 0.0_wp

          CASE ( 'wsss_r' )
             IF ( .NOT. ALLOCATED( wsss_res_av ) )  THEN
                ALLOCATE( wsss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             wsss_res_av = 0.0_wp

          CASE ( 'usss_s' )
             IF ( .NOT. ALLOCATED( usss_sgs_av ) )  THEN
                ALLOCATE( usss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             usss_sgs_av = 0.0_wp

          CASE ( 'vsss_s' )
             IF ( .NOT. ALLOCATED( vsss_sgs_av ) )  THEN
                ALLOCATE( vsss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             vsss_sgs_av = 0.0_wp

          CASE ( 'wsss_s' )
             IF ( .NOT. ALLOCATED( wsss_sgs_av ) )  THEN
                ALLOCATE( wsss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             wsss_sgs_av = 0.0_wp
!
!--       Balance terms
          CASE ( 'dsdt' )
             IF ( .NOT. ALLOCATED( dsdt_av ) )  THEN
                ALLOCATE( dsdt_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             dsdt_av = 0.0_wp
             
          CASE ( 'div' )
             IF ( .NOT. ALLOCATED( div_res_av ) )  THEN
                ALLOCATE( div_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             div_res_av = 0.0_wp             

          CASE ( 'dusss_r' )
             IF ( .NOT. ALLOCATED( d_usss_res_av ) )  THEN
                ALLOCATE( d_usss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_usss_res_av = 0.0_wp

          CASE ( 'dvsss_r' )
             IF ( .NOT. ALLOCATED( d_vsss_res_av ) )  THEN
                ALLOCATE( d_vsss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_vsss_res_av = 0.0_wp

          CASE ( 'dwsss_r' )
             IF ( .NOT. ALLOCATED( d_wsss_res_av ) )  THEN
                ALLOCATE( d_wsss_res_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_wsss_res_av = 0.0_wp

          CASE ( 'dusss_s' )
             IF ( .NOT. ALLOCATED( d_usss_sgs_av ) )  THEN
                ALLOCATE( d_usss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_usss_sgs_av = 0.0_wp

          CASE ( 'dvsss_s' )
             IF ( .NOT. ALLOCATED( d_vsss_sgs_av ) )  THEN
                ALLOCATE( d_vsss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_vsss_sgs_av = 0.0_wp

          CASE ( 'dwsss_s' )
             IF ( .NOT. ALLOCATED( d_wsss_sgs_av ) )  THEN
                ALLOCATE( d_wsss_sgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             d_wsss_sgs_av = 0.0_wp

          CASE ( 'uu' )
             IF ( .NOT. ALLOCATED( uu_av ) )  THEN
                ALLOCATE( uu_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             uu_av = 0.0_wp

          CASE ( 'vv' )
             IF ( .NOT. ALLOCATED( vv_av ) )  THEN
                ALLOCATE( vv_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             vv_av = 0.0_wp

          CASE ( 'ww' )
             IF ( .NOT. ALLOCATED( ww_av ) )  THEN
                ALLOCATE( ww_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ww_av = 0.0_wp

          CASE ( 'u_plus' )
             IF ( .NOT. ALLOCATED( u_plus_av ) )  THEN
                ALLOCATE( u_plus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             u_plus_av = 0.0_wp

          CASE ( 'v_plus' )
             IF ( .NOT. ALLOCATED( v_plus_av ) )  THEN
                ALLOCATE( v_plus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             v_plus_av = 0.0_wp

          CASE ( 'w_plus' )
             IF ( .NOT. ALLOCATED( w_plus_av ) )  THEN
                ALLOCATE( w_plus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             w_plus_av = 0.0_wp

          CASE ( 'u_minus' )
             IF ( .NOT. ALLOCATED( u_minus_av ) )  THEN
                ALLOCATE( u_minus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             u_minus_av = 0.0_wp

          CASE ( 'v_minus' )
             IF ( .NOT. ALLOCATED( v_minus_av ) )  THEN
                ALLOCATE( v_minus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             v_minus_av = 0.0_wp

          CASE ( 'w_minus' )
             IF ( .NOT. ALLOCATED( w_minus_av ) )  THEN
                ALLOCATE( w_minus_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             w_minus_av = 0.0_wp

          CASE ( 'ff' )
             IF ( .NOT. ALLOCATED( ff_av ) )  THEN
                ALLOCATE( ff_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ff_av = 0.0_wp

          CASE ( 'ff_h' )
             IF ( .NOT. ALLOCATED( ff_h_av ) )  THEN
                ALLOCATE( ff_h_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ff_h_av = 0.0_wp

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'sum' )  THEN

       SELECT CASE ( TRIM( variable ) )

          CASE ( 'usss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      usss_res_av(k,j,i) = usss_res_av(k,j,i) + usss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( usss_res_av, nbgp )

          CASE ( 'vsss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                     vsss_res_av(k,j,i) = vsss_res_av(k,j,i) + vsss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( vsss_res_av, nbgp )

          CASE ( 'wsss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      wsss_res_av(k,j,i) = wsss_res_av(k,j,i) + wsss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( wsss_res_av, nbgp )

          CASE ( 'usss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      usss_sgs_av(k,j,i) = usss_sgs_av(k,j,i) + usss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( usss_sgs_av, nbgp )

          CASE ( 'vsss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      vsss_sgs_av(k,j,i) = vsss_sgs_av(k,j,i) + vsss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( vsss_sgs_av, nbgp )

          CASE ( 'wsss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      wsss_sgs_av(k,j,i) = wsss_sgs_av(k,j,i) + wsss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( wsss_sgs_av, nbgp )
!
!--       Balance terms
          CASE ( 'dsdt' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      dsdt_av(k,j,i) = dsdt_av(k,j,i) + dsdt(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( dsdt_av, nbgp )
             
          CASE ( 'div' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      div_res_av(k,j,i) = div_res_av(k,j,i) + div_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( dsdt_av, nbgp )             


          CASE ( 'dusss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      d_usss_res_av(k,j,i) = d_usss_res_av(k,j,i) + d_usss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_usss_res_av, nbgp )

          CASE ( 'dvsss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                     d_vsss_res_av(k,j,i) = d_vsss_res_av(k,j,i) + d_vsss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_vsss_res_av, nbgp )

          CASE ( 'dwsss_r' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      d_wsss_res_av(k,j,i) = d_wsss_res_av(k,j,i) + d_wsss_res(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_wsss_res_av, nbgp )

          CASE ( 'dusss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      d_usss_sgs_av(k,j,i) = d_usss_sgs_av(k,j,i) + d_usss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_usss_sgs_av, nbgp )

          CASE ( 'dvsss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      d_vsss_sgs_av(k,j,i) = d_vsss_sgs_av(k,j,i) + d_vsss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_vsss_sgs_av, nbgp )

          CASE ( 'dwsss_s' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      d_wsss_sgs_av(k,j,i) = d_wsss_sgs_av(k,j,i) + d_wsss_sgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( d_wsss_sgs_av, nbgp )

          CASE ( 'uu' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      uu_av(k,j,i) = uu_av(k,j,i) + uu(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( uu_av, nbgp )

          CASE ( 'vv' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      vv_av(k,j,i) = vv_av(k,j,i) + vv(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( vv_av, nbgp )

          CASE ( 'ww' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      ww_av(k,j,i) = ww_av(k,j,i) + ww(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( ww_av, nbgp )

          CASE ( 'u_plus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      u_plus_av(k,j,i) = u_plus_av(k,j,i) + u_plus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( u_plus_av, nbgp )

          CASE ( 'v_plus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      v_plus_av(k,j,i) = v_plus_av(k,j,i) + v_plus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( v_plus_av, nbgp )

          CASE ( 'w_plus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      w_plus_av(k,j,i) = w_plus_av(k,j,i) + w_plus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( w_plus_av, nbgp )

          CASE ( 'u_minus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      u_minus_av(k,j,i) = u_minus_av(k,j,i) + u_minus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( u_minus_av, nbgp )

          CASE ( 'v_minus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      v_minus_av(k,j,i) = v_minus_av(k,j,i) + v_minus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( v_minus_av, nbgp )

          CASE ( 'w_minus' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      w_minus_av(k,j,i) = w_minus_av(k,j,i) + w_minus(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( w_minus_av, nbgp )

          CASE ( 'ff' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      ff_av(k,j,i) = ff_av(k,j,i) + ff(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( ff_av, nbgp )

          CASE ( 'ff_h' )
             DO  i = nxl, nxr
                DO  j = nys, nyn
                   DO  k = nzb, nzt+1
                      ff_h_av(k,j,i) = ff_h_av(k,j,i) + ff_h(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
             CALL exchange_horiz( ff_h_av, nbgp )

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'average' )  THEN

       SELECT CASE ( TRIM( variable ) )

          CASE ( 'usss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      usss_res_av(k,j,i) = usss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'vsss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      vsss_res_av(k,j,i) = vsss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'wsss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      wsss_res_av(k,j,i) = wsss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'usss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      usss_sgs_av(k,j,i) = usss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'vsss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      vsss_sgs_av(k,j,i) = vsss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'wsss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      wsss_sgs_av(k,j,i) = wsss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO
!
!--       Balance terms
          CASE ( 'dsdt' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      dsdt_av(k,j,i) = dsdt_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO
             
          CASE ( 'div' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      div_res_av(k,j,i) = div_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO             
             

          CASE ( 'dusss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_usss_res_av(k,j,i) = d_usss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'dvsss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_vsss_res_av(k,j,i) = d_vsss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'dwsss_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_wsss_res_av(k,j,i) = d_wsss_res_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'dusss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_usss_sgs_av(k,j,i) = d_usss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'dvsss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_vsss_sgs_av(k,j,i) = d_vsss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'dwsss_s' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      d_wsss_sgs_av(k,j,i) = d_wsss_sgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'uu' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      uu_av(k,j,i) = uu_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'vv' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      vv_av(k,j,i) = vv_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ww' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      ww_av(k,j,i) = ww_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'u_plus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      u_plus_av(k,j,i) = u_plus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'v_plus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      v_plus_av(k,j,i) = v_plus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'w_plus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      w_plus_av(k,j,i) = w_plus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'u_minus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      u_minus_av(k,j,i) = u_minus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'v_minus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      v_minus_av(k,j,i) = v_minus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'w_minus' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      w_minus_av(k,j,i) = w_minus_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ff' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      ff_av(k,j,i) = ff_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ff_h' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt+1
                      ff_h_av(k,j,i) = ff_h_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

       END SELECT

    ENDIF


 END SUBROUTINE user_3d_data_averaging
