!> @file user_data_output_mask.f90
!------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2017 Leibniz Universitaet Hannover
!------------------------------------------------------------------------------!
!
! Current revisions:
! ------------------
! 
! 
! Former revisions:
! -----------------
! $Id$
!
! 2000 2016-08-20 18:09:15Z knoop
! Forced header and separation lines into 80 columns
! 
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements, 
! kinds are defined in new module kinds, 
! old module precision_kind is removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! Description:
! ------------
!> Resorts the user-defined output quantity with indices (k,j,i) to a
!> temporary array with indices (i,j,k) for masked data output.
!------------------------------------------------------------------------------!
 SUBROUTINE user_data_output_mask( av, variable, found, local_pf )
 

    USE control_parameters
        
    USE indices
    
    USE kinds
    
    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  variable   !< 

    INTEGER(iwp) ::  av   !< 
    INTEGER(iwp) ::  i    !< 
    INTEGER(iwp) ::  j    !< 
    INTEGER(iwp) ::  k    !< 

    LOGICAL ::  found     !< 

    REAL(wp),                                                                  &
       DIMENSION(mask_size_l(mid,1),mask_size_l(mid,2),mask_size_l(mid,3)) ::  &
          local_pf   !< 


    found = .TRUE.

    SELECT CASE ( TRIM( variable ) )

       CASE ( 'usss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = usss_res(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = usss_res_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'vsss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vsss_res(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vsss_res_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'wsss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = wsss_res(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = wsss_res_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'usss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = usss_sgs(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = usss_sgs_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'vsss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vsss_sgs(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vsss_sgs_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'wsss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = wsss_sgs(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = wsss_sgs_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
!
!--    Balance terms
       CASE ( 'dsdt' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = dsdt(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = dsdt_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          
!
!--    Balance terms
       CASE ( 'div' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = div_res(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = div_res_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF


       CASE ( 'dusss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_usss_res(mask_k(mid,k),              &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_usss_res_av(mask_k(mid,k),           &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'dvsss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_vsss_res(mask_k(mid,k),              &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_vsss_res_av(mask_k(mid,k),           &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'dwsss_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_wsss_res(mask_k(mid,k),              &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_wsss_res_av(mask_k(mid,k),           &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'dusss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_usss_sgs(mask_k(mid,k),                &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_usss_sgs_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'dvsss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_vsss_sgs(mask_k(mid,k),              &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_vsss_sgs_av(mask_k(mid,k),           &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'dwsss_s' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_wsss_sgs(mask_k(mid,k),              &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = d_wsss_sgs_av(mask_k(mid,k),           &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'uu' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = uu(mask_k(mid,k),                      &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = uu_av(mask_k(mid,k),                   &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'vv' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vv(mask_k(mid,k),                      &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vv_av(mask_k(mid,k),                   &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'ww' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ww(mask_k(mid,k),                      &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ww_av(mask_k(mid,k),                   &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'u_plus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = u_plus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = u_plus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'v_plus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = v_plus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = v_plus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'w_plus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = w_plus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = w_plus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'u_minus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = u_minus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = u_minus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'v_minus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = v_minus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = v_minus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'w_minus' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = w_minus(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = w_minus_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'ff' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'ff_h' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_h(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_h_av(mask_k(mid,k),                 &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'ti' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ti(mask_k(mid,k),                    &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE DEFAULT
          found = .FALSE.

    END SELECT


 END SUBROUTINE user_data_output_mask
