!> @file user_module.f90
!------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2017 Leibniz Universitaet Hannover
!------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_module.f90 2101 2017-01-05 16:42:31Z suehring $
!
! 2000 2016-08-20 18:09:15Z knoop
! Forced header and separation lines into 80 columns
! 
! 1873 2016-04-18 14:50:06Z maronga
! Module renamed (removed _mod)
! 
! 
! 1850 2016-04-08 13:29:27Z maronga
! Module renamed
! 
! 
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements, 
! kinds are defined in new module kinds, 
! old module precision_kind is removed,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! Revision 1.1  1998/03/24 15:29:04  raasch
! Initial revision
!
!
! Description:
! ------------
!> Declaration of user-defined variables. This module may only be used
!> in the user-defined routines (contained in user_interface.f90).
!------------------------------------------------------------------------------!
 MODULE user
 

    USE kinds

    IMPLICIT NONE

    INTEGER(iwp) ::  dots_num_palm   !< 
    INTEGER(iwp) ::  user_idummy     !< 
    
    LOGICAL ::  user_defined_namelist_found = .FALSE.   !< 
    LOGICAL ::  residence = .FALSE.
    
    REAL(wp) ::  user_rdummy   !< 
!
!-- Flag indicating analysis volume to evaluate residental timescales 
!-- within courtyards    
    INTEGER(iwp), DIMENSION(:,:,:), ALLOCATABLE ::  analysis_flag
!
!-- Scalar source field (input only)
    REAL(wp), DIMENSION(:,:), ALLOCATABLE ::  s_source
!
!-- Time-averaged fluxes, resolved scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  usss_res_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vsss_res_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  wsss_res_av
!
!-- Time-averaged fluxes, subgrid scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  usss_sgs_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vsss_sgs_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  wsss_sgs_av
!
!-- Instantaneous fluxes, resolved scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  usss_res
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vsss_res
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  wsss_res
    
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  div_res
!
!-- Instantaneous fluxes, subgrid scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  usss_sgs
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vsss_sgs
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  wsss_sgs
!
!-- Instantaneous balance term, resolved scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_usss_res
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_vsss_res
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_wsss_res
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  dsdt
!
!-- Instantaneous balance term, subgrid scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_usss_sgs
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_vsss_sgs
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_wsss_sgs
!
!-- Time-averaged balance term, resolved scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_usss_res_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_vsss_res_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_wsss_res_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  dsdt_av
    
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  div_res_av
!
!-- Time-averaged balance term, subgrid scale
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_usss_sgs_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_vsss_sgs_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  d_wsss_sgs_av
!
!-- Time-averaged velocity fluxes, resolved scale only
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  uu_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vv_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ww_av
!
!-- Instantaneous velocity fluxes, resolved scale only
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  uu
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  vv
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ww
!
!-- Time-averaged positive and negative flux
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  u_plus_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  v_plus_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  w_plus_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  u_minus_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  v_minus_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  w_minus_av
!
!-- Instantaneous up- and downward flux
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  u_plus
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  v_plus
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  w_plus
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  u_minus
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  v_minus
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  w_minus
!
!-- turbulence intensity
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ti
!
!-- 3D wind velocity
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ff_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ff
!
!-- horizontal wind velocity
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ff_h_av
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  ff_h
!
!-- Dummy arrays, used to calculate balance term
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  s_previous

!
!-- Virtual measuring stations (vms)
    INTEGER(iwp) :: n_vms                   !< number of vms
    INTEGER(iwp) :: pos_vms(100,3)          !< i,j,k postions of vms

    REAL(iwp) :: pos_vms_x(100) = -9999  !< x postions of vms
    REAL(iwp) :: pos_vms_y(100) = -9999  !< y postions of vms
    REAL(iwp) :: pos_vms_z(100) = -9999  !< z postions of vms

    SAVE

 END MODULE user
